package tests.javaee;

import ejbs.MapKeyOrderByRemote;
import entities.Customer;
import entities.RetailOrder;
import entities.RetailOrderCostComparatorDESC;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import javax.naming.InitialContext;

public class TestOverrideOrderBy {
    
    private String testID;
    
    private static long ORDER_ID1 = 786l;
    private static long ORDER_ID2 = 787l;
    private static long ORDER_ID3 = 788l;
    private static long ORDER_ID4 = 789l;
    
    private static double COST1 = 100;
    private static double COST2 = 105;
    private static double COST3 = 110;
    private static double COST4 = 115;
    
    private static long CUST_ID1 = 2l;
    
    private List<RetailOrder> customer1sOrders;

    private MapKeyOrderByRemote remoteObj;
    
    public TestOverrideOrderBy() {
        testID = " TestOverrideOrderBy - JavaEE - ";
    }
    
    public void test() {
        try {
            remoteObj = (MapKeyOrderByRemote) new InitialContext().lookup("ejbs.MapKeyOrderByRemote");
            persistRelationship();
            assertRelationshipRetrievedInRightOrder();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void persistRelationship() {
        Customer customer1 = new Customer();
        customer1.setId(CUST_ID1);
        
        RetailOrder order115 = createOrder(ORDER_ID1, customer1, COST4);
        RetailOrder order100 = createOrder(ORDER_ID2, customer1, COST1);
        RetailOrder order110 = createOrder(ORDER_ID3, customer1, COST3);
        RetailOrder order105 = createOrder(ORDER_ID4, customer1, COST2);
        
        customer1sOrders = new ArrayList<RetailOrder>();
        customer1sOrders.add(order100);
        customer1sOrders.add(order115);
        customer1sOrders.add(order105);
        customer1sOrders.add(order110);
        
        customer1.setOrders(customer1sOrders);

        Set collectionToPersist = new HashSet();
        collectionToPersist.add(customer1);
        collectionToPersist.add(order115);
        collectionToPersist.add(order100);
        collectionToPersist.add(order110);
        collectionToPersist.add(order105);
        
        remoteObj.persistEntityCollection(collectionToPersist);
    }
    
    private void assertRelationshipRetrievedInRightOrder() throws Exception {
        // Order the list in descending order of cost !!
        Collections.sort(customer1sOrders, new RetailOrderCostComparatorDESC());
        
        Customer customer = (Customer) remoteObj.getEntity(Customer.class, CUST_ID1);
        List<RetailOrder> actualRetailOrders = customer.getOrders();
        
        for (RetailOrder order : actualRetailOrders){
            System.out.println("Order Id - " + order.getId());
        }
        
        if (customer1sOrders.equals(actualRetailOrders))
            System.out.println("Test - " + testID + " Passed");
        else{
            System.out.println("Test - " + testID + " Failed - The expected Orders List is not equal to the actual List read from the DB");
            printBothLists(customer1sOrders, actualRetailOrders);
        }
        
    }
    
    private RetailOrder createOrder(long id, Customer customer, double cost) {
        RetailOrder order = new RetailOrder();
        order.setId(id);
        order.addCustomer(customer);
        order.setCost(cost);
        return order;
    }
    
    private void printBothLists(List<RetailOrder> expectedList, List actualList) {
        System.out.println(testID + " Expected List - ");
        printList(expectedList);
        System.out.println(testID + " Actual List (from DB) - ");
        printList(actualList);
    }
    
    private void printList(List<RetailOrder> orderList) {
        for(RetailOrder order : orderList){
            System.out.println("Order ID - " + order.getId() + " Order Cost - " + order.getCost());
        }
    }
    
}
